@@ -2,7 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.6-3-90-1
+es5id: 15.2.3.6-3-90
 description: >
     Object.defineProperty - 'Attributes' is a Number object that uses
     Object's [[Get]] method to access the 'configurable' property
@@ -11,9 +11,10 @@ description: >
 
 var obj = {};
 
-Number.prototype.configurable = true;
 var numObj = new Number(-2);
 
+numObj.configurable = true;
+
 Object.defineProperty(obj, "property", numObj);
 
 var beforeDeleted = obj.hasOwnProperty("property");
